#include <ell/LatticeProteinPivotMoveStructState.hh>
#include <ell/CLatticeProteinPivotMoveStructState.hh>
#include <ell/assertell.hh>

namespace ell
{

	const LatticeModel*
	 LatticeProteinPivotMoveStructState::commonLattice = NULL;
	const ContactEnergyFunction*
	 LatticeProteinPivotMoveStructState::commonEnergy = NULL;	
	std::string LatticeProteinPivotMoveStructState::commonSeqStr;
//////////////////////////////////////////////////////////
// LatticeProteinPivotMoveStructState
//////////////////////////////////////////////////////////

	LatticeProteinPivotMoveStructState::LatticeProteinPivotMoveStructState(
	 const std::string& moveString, const bool isAbsoluteMove)
	 : LatticeProteinPivotMoveState(commonLattice, commonEnergy,
		commonSeqStr, moveString, isAbsoluteMove)
	{}

	LatticeProteinPivotMoveStructState::LatticeProteinPivotMoveStructState(
				const LatticeProteinPivotMoveStructState& latProtPMState2)
	 : LatticeProteinPivotMoveState(latProtPMState2)
	{}

	LatticeProteinPivotMoveStructState::~LatticeProteinPivotMoveStructState() {
	}

	void
	LatticeProteinPivotMoveStructState::setCommonLatticeModel(
	 const LatticeModel* lattice) {
		assertell ( commonLattice == NULL,
		"It is not allowed to set the common lattice model more than once.");
		commonLattice = lattice;
	}

	void
	LatticeProteinPivotMoveStructState::setCommonContactEnergyFunction(
	 const ContactEnergyFunction* energy) {
		assertell ( commonEnergy == NULL,
		"It is not allowed to set the common energy function more than once.");
		commonEnergy = energy;
	}

	void
	LatticeProteinPivotMoveStructState::setCommonSequenceString(
	 const std::string& seqStr) {
		assertell ( commonSeqStr.empty(),
		"It is not allowed to set the common sequence string more than once.");
		commonSeqStr = seqStr;
	}

	LatticeProteinPivotMoveStructState* 
	LatticeProteinPivotMoveStructState::clone() const {
		return new LatticeProteinPivotMoveStructState(*this);
	}

	CState*
	LatticeProteinPivotMoveStructState::compressedClone() const {
		return new CLatticeProteinPivotMoveStructState(
		 latProt.getStructure());
	}

	void
	LatticeProteinPivotMoveStructState::compress(CState& cState) const {
		dynamic_cast<CLatticeProteinPivotMoveStructState&>(cState)
		 = CLatticeProteinPivotMoveStructState(
			latProt.getStructure());
	}

	State::NeighborListPtr 
	LatticeProteinPivotMoveStructState::getNeighborList() const {
		return NeighborListPtr(new NeighborList(this));
	}

	State::NeighborListPtr
	LatticeProteinPivotMoveStructState::getRandomNeighborList()
				const {
		return NeighborListPtr(new RandomNeighborList(this));
	}

	// NeighborList
	LatticeProteinPivotMoveStructState::NeighborList::NeighborList(
	 			const LatticeProteinPivotMoveStructState* _origin) 
	 : LatticeProteinPivotMoveState::NeighborList(_origin), origin(_origin)
	{
	}

	State* 
	LatticeProteinPivotMoveStructState::NeighborList::first(
		      State::NeighborList::ItState** itstate) const {
		if (*itstate != NULL) delete *itstate;

		ItState* myitstate = new ItState();
		*itstate = myitstate;

		LatticeProteinPivotMoveStructState* latProtState = NULL;

		PivotMove firstMove = origin->firstPivotMove();

		if (firstMove != PivotMove(0,INVALID_INDEX)) {
			latProtState = new LatticeProteinPivotMoveStructState(*origin);
			latProtState->pivotMoveSet.applyMoveInPlace(&latProtState->latProt,
			 firstMove.first, firstMove.second);
			myitstate->lastMove = firstMove;
				// if applied pivot move generates an invalid neighbor,
				// call next(..) to iterate neighbors until a VALID one is
				// found which will be returned
			if (!(latProtState->latProt.isValid())) {
				return next(*itstate, latProtState);
			}
		}

		return latProtState;
	}

	State* 
	LatticeProteinPivotMoveStructState::NeighborList::next(
		      State::NeighborList::ItState* itstate, State* elem) const {
		assertell(elem != NULL, "Elem is not allowed to be NULL");

		ItState* myitstate = static_cast<ItState*>(itstate);

			// find next move
		PivotMove nextMove = origin->nextPivotMove(myitstate->lastMove);
			// no more moves possible
		if (nextMove == PivotMove(0,INVALID_INDEX)) {
			return NULL;
		}
		else {
			LatticeProteinPivotMoveStructState* latProtState =
			 dynamic_cast<LatticeProteinPivotMoveStructState*>(elem);
				// set latProtState to the state of origin of the neighbors
			latProtState->latProt = origin->latProt;
				// perform next move
			latProtState->pivotMoveSet.applyMoveInPlace(&latProtState->latProt,
			 nextMove.first, nextMove.second);
			myitstate->lastMove = nextMove;
				// if applied pivot move generates an invalid neighbor,
				// call next(..) to iterate neighbors until a VALID one is
				// found which will be returned
			if (!(latProtState->latProt.isValid())) {
				return next(itstate, latProtState);
			}
			return latProtState;
		}
	}

	// RandomNeighborList
	LatticeProteinPivotMoveStructState::RandomNeighborList::RandomNeighborList(
	 		const LatticeProteinPivotMoveStructState* _origin)
	 : LatticeProteinPivotMoveState::RandomNeighborList(_origin),
	   origin(_origin) 
	{
	}

	State* 
	LatticeProteinPivotMoveStructState::RandomNeighborList::first(
		      State::NeighborList::ItState** itstate) const {
		if (*itstate != NULL) delete *itstate;

		*itstate = new ItState();
		LatticeProteinPivotMoveStructState* latProtState = 
		 new LatticeProteinPivotMoveStructState(*origin);

		return next(*itstate, latProtState);
	}

	State* 
	LatticeProteinPivotMoveStructState::RandomNeighborList::next(
		      State::NeighborList::ItState* itstate, State* elem) const {
		assertell(elem != NULL, "Elem is not allowed to be NULL");

		ItState* myitstate = static_cast<ItState*>(itstate);
		LatticeProteinPivotMoveStructState* latProtState =
		 dynamic_cast<LatticeProteinPivotMoveStructState*>(elem);

		PivotMove nextMove;

		if (myitstate->switchModeValue > 0) {
				// position and index of the next pivot move
			size_t pos, moveIndex;
			bool foundValidMove = false;

			while (!foundValidMove) {
					// move position has to be in [1, structure length)
				pos = (random() % (origin->latProt.getStructure().size()-1))+1;
					// moveIndex has to be in [0, number of available moves)
				moveIndex = random() % origin->pivotMoveSet.getMoveNumber();

				nextMove = PivotMove(pos, moveIndex);
					// move has already been chosen
				if (!myitstate->chosenMoves.insert(nextMove).second) {
					myitstate->switchModeValue--;
					if (myitstate->switchModeValue == 0) {
						switchMode(myitstate);
						break;
					}
				}
					// move hasn't been chosen yet
				else {
						// set latProtState to the state of origin of the
						// neighbors
					latProtState->latProt = origin->latProt;
					latProtState->pivotMoveSet.applyMoveInPlace(&latProtState->latProt,
					 nextMove.first, nextMove.second);
						// if applied pivot move generates an invalid neighbor,
						// iterate neighbors until a VALID one is found which
						// will be returned
					foundValidMove = latProtState->latProt.isValid();
				}	
			}
		}
		if (myitstate->switchModeValue == 0) {
			size_t n = myitstate->unchosenMoves.size();
			if (n == 0) {
				return NULL;
			}
			else {
				size_t i = random() % n;
					// set latProtState to the state of origin of the neighbors
				latProtState->latProt = origin->latProt;
				nextMove = myitstate->unchosenMoves[i];
				latProtState->pivotMoveSet.applyMoveInPlace(&latProtState->latProt,
				 nextMove.first, nextMove.second);
				myitstate->unchosenMoves[i] =
				 myitstate->unchosenMoves[n-1];
				myitstate->unchosenMoves.resize(n-1);
					// if applied pivot move generates an invalid neighbor,
					// call next(..) to iterate neighbors until a VALID one is
					// found which will be returned
				if (!(latProtState->latProt.isValid())) {
					return next(itstate, latProtState);
				}
			}
		}
		return latProtState;
	}

} // namespace ell
