#include <ell/PivotMoveSet.hh>

namespace ell
{
	PivotMoveSet::PivotMoveSet(const LatticeModel* lattice)
	 :	LatticeMoveSet(lattice) , 
	 	lastOverwrittenMove(lattice->getNeighborhood().size()-1),
	 	lastOverwritePos(0),
	 	lastChangedObject(NULL)
	{} 

	PivotMoveSet::~PivotMoveSet()
	{}

// abstract functions (LatticeMoveSet)
	LatticeProtein*
	PivotMoveSet::applyMove(const LatticeProtein* const todo, 
	 size_t position, size_t moveIndex)  {

			// apply the pivot move inplace to a copy of todo
		return applyMoveInPlace(new LatticeProtein(*todo), position,
				 moveIndex);
	}

	LatticeProtein* 
	PivotMoveSet::applyMoveInPlace(LatticeProtein* todo, 
	 size_t position, size_t moveIndex)  {

		assertell(moveIndex >=0 && moveIndex < getMoveNumber(),
	"The pivot move moveIndex has to be in [0, number of available moves).");
		assertell(position >0 && position < todo->relMoves->size(),
	"The pivot move position has to be in [1, structure length).");


			// get the mutated relative move
		Move mutatedMove = lattice->getNeighborhood().getElementByIndex(moveIndex).getMove();

			
			// if the current relative move at 'position' is equal to the chosen
			// mutated move, choose the move with the highest move index
			// this one is not chosen randomly
			// by that it is ensured that always a random neighbor is returned
		if ((*(todo->relMoves))[position] == mutatedMove) {
			mutatedMove = lattice->getNeighborhood().getElementByIndex(getMoveNumber()).getMove();
		}

		lastChangedObject = NULL;

		if ( todo->relMoves->at(position) != mutatedMove) {
				// save for next undo
			lastOverwrittenMove	= todo->relMoves->at(position);
			lastOverwritePos 	= position;
			lastChangedObject	= todo;
				// remove current point coordinates
			todo->clearPoints();
				// change the relative move string at position to the mutated move
			todo->relMoves->at(position) = mutatedMove;
		}


		return todo;
	}

	unsigned int
	PivotMoveSet::getMoveNumber() const {
			// number of pivot moves is the number of relative moves minus one
			// for the current relative move
		return lattice->getNeighborhood().size() - 1;
	}


	LatticeProtein* 
	PivotMoveSet::undoLastMove(LatticeProtein* toUndo) {
		if (toUndo == lastChangedObject 
			&& (*(toUndo->relMoves))[lastOverwritePos] != lastOverwrittenMove) 
		{
			(*(toUndo->relMoves))[lastOverwritePos] = lastOverwrittenMove;
			toUndo->clearPoints();
		}
		
		return toUndo;
	}

}
